﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("SumAccuracy", "")]
  [StorableClass]
  public class SumAccuracy : SingleSuccessorOperator {
    public ILookupParameter<ItemArray<DoubleValue>> AccuracyParameter {
      get { return (ILookupParameter<ItemArray<DoubleValue>>)Parameters["Accuracy"]; }
    }
    public ILookupParameter<ItemArray<IntValue>> NumerosityParameter {
      get { return (ILookupParameter<ItemArray<IntValue>>)Parameters["Numerosity"]; }
    }
    protected IValueLookupParameter<DoubleValue> AccuracySumParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["AccuracySum"]; }
    }

    [StorableConstructor]
    protected SumAccuracy(bool deserializing) : base(deserializing) { }
    protected SumAccuracy(SumAccuracy original, Cloner cloner)
      : base(original, cloner) {
    }
    public SumAccuracy()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("Accuracy"));
      Parameters.Add(new ScopeTreeLookupParameter<IntValue>("Numerosity"));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("AccuracySum"));
      Parameters.Add(new ScopeParameter("CurrentScope", "The current scope from which sub-scopes should be selected."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new SumAccuracy(this, cloner);
    }

    public sealed override IOperation Apply() {
      ItemArray<DoubleValue> accuracies = AccuracyParameter.ActualValue;
      ItemArray<IntValue> numerosities = NumerosityParameter.ActualValue;
      double accuracySum = 0.0;
      for (int i = 0; i < accuracies.Length; i++) {
        accuracySum += accuracies[i].Value * numerosities[i].Value;
      }
      AccuracySumParameter.ActualValue = new DoubleValue(accuracySum);
      return base.Apply();
    }
  }
}
